@@ -13,6 +13,7 @@
 #include <avogadro/core/dihedraliterator.h>
 #include <avogadro/core/elements.h>
 #include <avogadro/core/molecule.h>
+#include <avogadro/core/unitcell.h>
 
 namespace Avogadro::Calc {
 
@@ -360,7 +361,8 @@ public:
       a._c0 = 0.0;
       a._c1 = 0.0;
       a._c2 = 0.0;
-      if (label.size() < 3 || neighbors.size() == 1) {
+
+      if (label.size() < 3 || neighbors.size() == 1 || label[2] == '1') {
         // linear
         a.coordination = Linear;
         a._c0 = 1.0;
@@ -680,10 +682,12 @@ public:
       Real theta = acos(dot / (r1 * r2));
 
       /*
-      std::cout << " Angle " << i << " " << j << " " << k << " " << r1 << " "
-                << r2 << " " << theta * RAD_TO_DEG << " " << theta0 * RAD_TO_DEG
-                << std::endl;
-  */
+            std::cout << " Angle " << angle.coordination << " " << i << " " << j
+         << " " << k << " " << r1 << " "
+                      << r2 << " " << theta * RAD_TO_DEG << " " << theta0 *
+         RAD_TO_DEG
+                      << std::endl;
+      */
 
       // TODO - migrate special cases from Open Babel
       Coordination coord = angle.coordination;
@@ -794,12 +798,17 @@ public:
       Real x6 = xij * xij * xij * xij * xij * xij;
       Real x12 = x6 * x6;
 
-      // TODO: use the unit cell if available
-      Real dx = x[3 * i] - x[3 * j];
-      Real dy = x[3 * i + 1] - x[3 * j + 1];
-      Real dz = x[3 * i + 2] - x[3 * j + 2];
+      Vector3 atom_i(x[3 * i], x[3 * i + 1], x[3 * i + 2]);
+      Vector3 atom_j(x[3 * j], x[3 * j + 1], x[3 * j + 2]);
+      // if the cell is nullptr, we can't do periodic boundary conditions
+      Real r2;
+      if (m_cell == nullptr) {
+        r2 = (atom_i - atom_j).squaredNorm();
+      } else {
+        r2 = m_cell->distanceSquared(atom_i, atom_j);
+      }
+
       // we don't need a square root since 6 and 12 are even powers
-      Real r2 = (dx * dx + dy * dy + dz * dz);
       Real r6 = r2 * r2 * r2;
       Real r12 = r6 * r6;
       energy += depth * (x12 / r12 - 2 * x6 / r6);
@@ -861,7 +870,7 @@ public:
       Real crossNorm = ij_cross_kj.norm();
 
       // check for near-zero cross product
-      if (crossNorm < 1e-6)
+      if (!std::isfinite(crossNorm) || crossNorm < 1e-6)
         continue; // skip this angle
 
       Real theta = atan2(crossNorm, dot);
@@ -889,7 +898,7 @@ public:
         case Linear:
           // fixed typo in UFF paper (it's 1+ cos(theta) not 1 - cos(theta))
           // energy += kijk * (1 + cos(c0 * theta));
-          f = kijk * c0 * sin(c0 * theta);
+          f = -kijk * c0 * sin(c0 * theta);
           break;
         case Trigonal:
         case Resonant:
@@ -902,8 +911,8 @@ public:
           // energy +=
           // kijk * (1 - cos(c0 * theta)) + exp(-20.0 * (theta - theta0 +
           // 0.25));
-          f = kijk * c0 * sin(c0 * theta) +
-              20.0 * exp(-20.0 * (theta - theta0 + 0.25)) * sin(theta);
+          f = kijk * c0 * sin(c0 * theta) -
+              20.0 * exp(-20.0 * (theta - theta0 + 0.25));
 
           break;
         case Tetrahedral: {
@@ -1186,11 +1195,18 @@ public:
       //         = -12 * depth * x^6 / r^7 * (x^6 / r^6 - 1)
 
       // TODO: handle unit cells and periodic boundary conditions
-      Real dx = x[3 * i] - x[3 * j];
-      Real dy = x[3 * i + 1] - x[3 * j + 1];
-      Real dz = x[3 * i + 2] - x[3 * j + 2];
+      Vector3 atom_i(x[3 * i], x[3 * i + 1], x[3 * i + 2]);
+      Vector3 atom_j(x[3 * j], x[3 * j + 1], x[3 * j + 2]);
+      Vector3 r = atom_i - atom_j;
+      if (m_cell != nullptr) {
+        r = m_cell->minimumImage(r);
+      }
+      Real r2 = r.squaredNorm();
+
+      Real dx = r[0];
+      Real dy = r[1];
+      Real dz = r[2];
 
-      Real r2 = dx * dx + dy * dy + dz * dz;
       Real r6 = r2 * r2 * r2;
       Real r7 = r6 * sqrt(r2);
       Real x6 = xij * xij * xij * xij * xij * xij;
