@@ -172,7 +172,7 @@ bool MdlFormat::read(std::istream& in, Core::Molecule& mol)
     }
 
     string element(trimmed(buffer.substr(31, 3)));
-    auto charge(lexicalCast<int>(trimmed(buffer.substr(36, 3))));
+    auto charge(lexicalCast<int>(trimmed(buffer.substr(36, 3))).value_or(0));
     if (!buffer.empty()) {
       unsigned char atomicNum = Elements::atomicNumberFromSymbol(element);
       Atom newAtom = mol.addAtom(atomicNum);
@@ -233,9 +233,6 @@ bool MdlFormat::read(std::istream& in, Core::Molecule& mol)
     if (!in.good() || buffer.size() < 6) {
       break;
     }
-#ifndef NDEBUG
-    std::cout << " prefix " << buffer.substr(0, 6) << std::endl;
-#endif
 
     string prefix = buffer.substr(0, 6);
     if (prefix == "M  END") {
@@ -323,6 +320,11 @@ bool MdlFormat::read(std::istream& in, Core::Molecule& mol)
         // TODO: Implement isotope setting
         // mol.atom(index).setIsotope(isotope);
       }
+    } // isotope
+    else {
+#ifndef NDEBUG
+      std::cout << " prefix " << buffer.substr(0, 6) << std::endl;
+#endif
     }
   }
 
@@ -409,7 +411,7 @@ bool MdlFormat::read(std::istream& in, Core::Molecule& mol)
     // convert the idx to an atom index
     // and set the label
     std::string idx = mol.data("idx").toString();
-    size_t atomIdx = lexicalCast<size_t>(idx);
+    size_t atomIdx = lexicalCast<size_t>(idx).value_or(0);
     mol.setAtomLabel(atomIdx, pka);
   }
 
@@ -519,6 +521,10 @@ bool MdlFormat::readV3000(std::istream& in, Core::Molecule& mol)
           }
           // TODO: handle isotopes
           // mol.atom(i).setIsotope(isotope);
+        } else {
+#ifndef NDEBUG
+          std::cerr << "Unknown key: " << key << std::endl;
+#endif
         }
       } // end of key-value loop
     }
