@@ -28,6 +28,7 @@ namespace Avogadro::Io {
 using Core::Atom;
 using Core::Elements;
 using Core::lexicalCast;
+using Core::rstrip;
 using Core::split;
 
 #ifndef _WIN32
@@ -59,23 +60,34 @@ bool TurbomoleFormat::read(std::istream& inStream, Core::Molecule& mol)
   string buffer;
   getline(inStream, buffer);
   while (inStream.good() && !buffer.empty()) {
-    if (buffer.find("$end") != std::string::npos)
+    std::vector<string> tokens = split(rstrip(buffer, '#'), ' ');
+    if (tokens.empty()) { // "# comment line"
+      getline(inStream, buffer);
+      continue;
+    }
+
+    if (tokens[0] == "$end")
       break;
-    else if (buffer.find("$coord") != std::string::npos) {
+
+    if (tokens[0] == "$coord") {
       // check if there's a conversion to be done
       Real coordConversion = BOHR_TO_ANGSTROM; // default is Bohr
-      if (buffer.find("ang") != std::string::npos)
+      if (std::find(tokens.begin(), tokens.end(), "ang") != tokens.end())
         coordConversion = 1.0; // leave as Angstrom
-      else if (buffer.find("frac") != std::string::npos) {
+      else if (std::find(tokens.begin(), tokens.end(), "frac") !=
+               tokens.end()) {
         fractionalCoords = true;
         coordConversion = 1.0; // we may not know the lattice constants yet
+      } else if (tokens.size() > 1u && tokens[1][0] != '#') {
+        std::cerr << "Ignore unknown trailing token and assume bohr: " << buffer
+                  << '\n';
       }
 
       getline(inStream, buffer);
-      while (buffer.find("$") == std::string::npos) {
+      tokens = split(rstrip(buffer, '#'), ' ');
+      while (!tokens.empty() && tokens[0][0] != '$') {
         // parse atoms until we see another '$' section
         // e.g. 0.0000      0.000000     -0.73578      o
-        std::vector<string> tokens(split(buffer, ' '));
 
         if (tokens.size() < 4) {
           appendError("Not enough tokens in this line: " + buffer);
@@ -87,64 +99,84 @@ bool TurbomoleFormat::read(std::istream& inStream, Core::Molecule& mol)
           tokens[3][0] = toupper(tokens[3][0]);
           atomicNum = Elements::atomicNumberFromSymbol(tokens[3]);
         } else
-          atomicNum =
-            static_cast<unsigned char>(lexicalCast<short int>(tokens[3]));
+          atomicNum = static_cast<unsigned char>(
+            lexicalCast<short int>(tokens[3]).value_or(0));
 
-        Vector3 pos(lexicalCast<double>(tokens[0]),
-                    lexicalCast<double>(tokens[1]),
-                    lexicalCast<double>(tokens[2]));
+        Vector3 pos;
+        if (auto tmp =
+              lexicalCast<double>(tokens.begin(), tokens.begin() + 3)) {
+          pos << tmp->at(0), tmp->at(1), tmp->at(2);
+        } else {
+          appendError("Failed to parse this line following $coord: " + buffer);
+          return false;
+        }
 
         Atom newAtom = mol.addAtom(atomicNum);
         newAtom.setPosition3d(pos * coordConversion);
 
         // next line
         getline(inStream, buffer);
+        tokens = split(rstrip(buffer, '#'), ' ');
       }
-    } else if (buffer.find("$cell") != std::string::npos) {
+    } else if (tokens[0] == "$cell") {
       hasCell = true;
       Real cellConversion = BOHR_TO_ANGSTROM;
-      if (buffer.find("ang") != std::string::npos)
+      if (std::find(tokens.begin(), tokens.end(), "angs") != tokens.end())
         cellConversion = 1.0; // leave as Angstrom
 
       getline(inStream, buffer);
-      std::vector<string> tokens(split(buffer, ' '));
+      tokens = split(rstrip(buffer, '#'), ' ');
       if (tokens.size() < 6) {
         appendError("Not enough tokens in this line: " + buffer);
         return false;
       }
-      a = lexicalCast<double>(tokens[0]) * cellConversion;
-      b = lexicalCast<double>(tokens[1]) * cellConversion;
-      c = lexicalCast<double>(tokens[2]) * cellConversion;
-      alpha = lexicalCast<double>(tokens[3]) * DEG_TO_RAD;
-      beta = lexicalCast<double>(tokens[4]) * DEG_TO_RAD;
-      gamma = lexicalCast<double>(tokens[5]) * DEG_TO_RAD;
+      if (auto tmp = lexicalCast<double>(tokens.begin(), tokens.begin() + 6)) {
+        a = tmp->at(0) * cellConversion;
+        b = tmp->at(1) * cellConversion;
+        c = tmp->at(2) * cellConversion;
+        alpha = tmp->at(3) * DEG_TO_RAD;
+        beta = tmp->at(4) * DEG_TO_RAD;
+        gamma = tmp->at(5) * DEG_TO_RAD;
+      } else {
+        appendError("Failed to parse this line: " + buffer);
+        return false;
+      }
 
-    } else if (buffer.find("$lattice") != std::string::npos) {
+    } else if (tokens[0] == "$lattice") {
       hasLattice = true;
       Real latticeConversion = BOHR_TO_ANGSTROM; // default
-      if (buffer.find("ang") != std::string::npos)
+      if (std::find(tokens.begin(), tokens.end(), "angs") != tokens.end())
         latticeConversion = 1.0; // leave as Angstrom
 
       for (int line = 0; line < 3; ++line) {
         getline(inStream, buffer);
-        std::vector<string> tokens(split(buffer, ' '));
+        tokens = split(rstrip(buffer, '#'), ' ');
         if (tokens.size() < 3)
           break;
 
-        if (line == 0) {
-          v1.x() = lexicalCast<double>(tokens[0]) * latticeConversion;
-          v1.y() = lexicalCast<double>(tokens[1]) * latticeConversion;
-          v1.z() = lexicalCast<double>(tokens[2]) * latticeConversion;
-        } else if (line == 1) {
-          v2.x() = lexicalCast<double>(tokens[0]) * latticeConversion;
-          v2.y() = lexicalCast<double>(tokens[1]) * latticeConversion;
-          v2.z() = lexicalCast<double>(tokens[2]) * latticeConversion;
-        } else if (line == 2) {
-          v3.x() = lexicalCast<double>(tokens[0]) * latticeConversion;
-          v3.y() = lexicalCast<double>(tokens[1]) * latticeConversion;
-          v3.z() = lexicalCast<double>(tokens[2]) * latticeConversion;
+        if (auto tmp =
+              lexicalCast<double>(tokens.begin(), tokens.begin() + 3)) {
+          if (line == 0) {
+            v1.x() = tmp->at(0) * latticeConversion;
+            v1.y() = tmp->at(1) * latticeConversion;
+            v1.z() = tmp->at(2) * latticeConversion;
+          } else if (line == 1) {
+            v2.x() = tmp->at(0) * latticeConversion;
+            v2.y() = tmp->at(1) * latticeConversion;
+            v2.z() = tmp->at(2) * latticeConversion;
+          } else if (line == 2) {
+            v3.x() = tmp->at(0) * latticeConversion;
+            v3.y() = tmp->at(1) * latticeConversion;
+            v3.z() = tmp->at(2) * latticeConversion;
+          }
+        } else {
+          appendError("Failed to parse this line following $lattice: " +
+                      buffer);
+          return false;
         }
       }
+    } else if (tokens[0][0] != '#') {
+      std::cerr << "Ignore unknown token: " << buffer << '\n';
     }
 
     getline(inStream, buffer);
