@@ -130,6 +130,25 @@ void LineFormatInput::showDialog()
 
   QtGui::Molecule newMol;
   m_reader->readString(m_descriptor, newMol);
+
+  // check to make sure the coordinates of newMol are non-zero
+  bool allZero = true;
+  for (Index i = 0; i < newMol.atomCount(); i++) {
+    auto vector = newMol.atomPosition3d(i);
+    if (vector.x() != 0.0 || vector.y() != 0.0 || vector.z() != 0.0) {
+      allZero = false;
+      break;
+    }
+  }
+
+  if (allZero) {
+    QMessageBox::warning(parentAsWidget, tr("No coordinates found!"),
+                         tr("Unable to generate 3D coordinates."),
+                         QMessageBox::Ok);
+    delete m_reader;
+    return;
+  }
+
   m_molecule->undoMolecule()->appendMolecule(newMol, "Insert Molecule");
   emit requestActiveTool("Manipulator");
   dlg.hide();
