@@ -11,8 +11,8 @@
 #include <avogadro/core/crystaltools.h>
 #include <avogadro/core/unitcell.h>
 #include <avogadro/qtgui/molecule.h>
-#include <avogadro/vtk/chartdialog.h>
-#include <avogadro/vtk/chartwidget.h>
+#include <avogadro/qtgui/chartdialog.h>
+#include <avogadro/qtgui/chartwidget.h>
 
 #include "pdfoptionsdialog.h"
 #include "plotpdf.h"
@@ -136,12 +136,12 @@ void PlotPdf::displayDialog()
 
   if (!m_chartDialog)
     m_chartDialog.reset(
-      new VTK::ChartDialog(qobject_cast<QWidget*>(this->parent())));
+      new QtGui::ChartDialog(qobject_cast<QWidget*>(this->parent())));
 
   m_chartDialog->setWindowTitle(windowName);
   auto* chart = m_chartDialog->chartWidget();
   chart->clearPlots();
-  chart->addPlot(xData, yData, VTK::color4ub{ 255, 0, 0, 255 });
+  chart->addPlot(xData, yData, QtGui::color4ub{ 255, 0, 0, 255 });
   chart->setXAxisTitle(xTitle);
   chart->setYAxisTitle(yTitle);
   m_chartDialog->show();
