@@ -28,7 +28,7 @@ QTAIMODEIntegrator::QTAIMODEIntegrator(QTAIMWavefunctionEvaluator& eval,
                                        const qint64 mode)
   : m_eval(&eval), m_mode(mode), m_associatedSphere(0)
 {
-  m_betaSpheres.empty();
+  m_betaSpheres.clear();
 }
 
 QVector3D QTAIMODEIntegrator::integrate(QVector3D x0y0z0)
