@@ -11,8 +11,7 @@
 
 #include <avogadro/core/vector.h>
 
-namespace Avogadro {
-namespace QtPlugins {
+namespace Avogadro::QtPlugins {
 
 typedef struct
 {
@@ -34,7 +33,6 @@ public:
                            QVector<specialKPoint>& specialKPoints);
 };
 
-} // namespace QtPlugins
-} // namespace Avogadro
+} // namespace Avogadro::QtPlugins
 
 #endif // AVOGADRO_QTPLUGINS_YAEHMOPOUT_H
