@@ -7,7 +7,7 @@
 HERE="$(dirname "$(readlink -f "${0}")")"
 
 export LD_LIBRARY_PATH="${HERE}/usr/lib:${LD_LIBRARY_PATH}"
-export BABEL_DATADIR=$(readlink -f "${HERE}/usr/share/openbabel/*/")
-export BABEL_LIBDIR=$(readlink -f "${HERE}/usr/lib/openbabel/")
+#export BABEL_DATADIR=$(readlink -f "${HERE}/usr/share/openbabel/*/")
+#export BABEL_LIBDIR=$(readlink -f "${HERE}/usr/lib/openbabel/")
 
-exec "${HERE}/usr/bin/avogadro2" "$@"
\ No newline at end of file
+exec "${HERE}/usr/bin/avogadro2" "$@"
